==> etherx-0.8.tar.gz: etherx-0.8/daemon/netout.c <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherxd.h"

ehandler connections = NULL;

/* this is a simple handler for net connections etherx has made to others */
void netout_onNode(thread t, xmlnode in, int type)
{
    tstream ts = (tstream)(t->data);
    ehandler eh = (ehandler)(ts->data_aux1);

    if(type == XMLSTREAM_INIT)
    {
        pth_cleanup_push(netout_cleanup, (void *)t);
        return;
    }

    if(type == XMLSTREAM_ROOT)
    {
        if(j_strcmp(xmlnode_get_attrib(in, "xmlns"), eh->namespace) != 0)
            xmlstream_error(t,"Invalid Namespace.");
        return;
    }

    if(type == XMLSTREAM_ERROR)
    {
        xmlstream_error(t,"You sent invalid XML.");
        return;
    }

    if(type == XMLSTREAM_NODE)
    {
        if(j_strcmp(xmlnode_get_name(in),"stream:error") == 0)
        {
            jerror("etherxd","remote server XML Stream error: %s",xmlnode_get_data(xmlnode_get_firstchild(in)));
            pth_exit(NULL);
            return;
        }
        jwarn("etherxd","receiving data on connection to remote server where we shouldn't be");
    }
}

void netout_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    ehandler eh = (ehandler)(ts->data_aux1);

    jdebug(ZONE,"netout_cleanup");

    logger_conn2(eh->namespace, eh->host, ts->flag_aux1);
    connections = ehandler_list_free(connections, eh);
}

void netout_write(thread t, epacket e)
{
    tstream ts = (tstream)(t->data);
    tstream_buffer b;
 
    jdebug(ZONE,"net_write");
 
    if(t == NULL || e == NULL)
        return;
 
    b = pmalloc(xmlnode_pool(e->x),sizeof(struct tstream_buffer_struct));
    b->data = xmlnode2str(epacket_serialize(e));
    b->data_sz = strlen(b->data);
    b->sent = 0;
    b->obj = (void *)e;
    b->p = xmlnode_pool(e->x);

    if(!ts->flag1) /* store out of band if the stream hasn't fully opened both ways yet */
        tstream_write_buffer(t, b);
    else
        tstream_oob_buffer(t, b);
}

void netout_error(thread t, tstream_buffer buffer)
{
    /* the stream is closed and these are the epackets that didn't get sent */
    epacket e, eold = (epacket)(buffer->obj);
 
    jdebug(ZONE,"net_error");
 
    if(buffer != NULL && eold != NULL)
    {
        e = epacket_new(xmlnode_dup(eold->x),eold->to, eold->from, eold->ns);
        jerror("etherxd","bouncing data intended for %s",e->to);
        epacket_error(e,"delivery failure");
        manager_send(e);
    }
}

ehandler netout_connect(char *host, char *namespace)
{
    ehandler eh;
    tstream ts;

    eh = _ehandler_new(host, namespace);

    /* create a special new connection! xmlstream can parse it, but we'll handle the buffer */
    eh->t =  tstream_connect(5269, host, _xmlstream_main, netout_error, netout_onNode);
    xmlstream_write_header(eh->t, xmlstream_header(namespace, host, NULL));
    ts = (tstream)(eh->t->data);
    ts->data_aux1 = (void *)eh;

    connections = ehandler_list_add(connections, eh);
}

==> etherx-0.9pre3.tar.gz: etherx-0.9pre3/daemon/netout.c <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherxd.h"

ehandler connections = NULL;

/* this is a simple handler for net connections etherx has made to others */
void netout_onNode(thread t, xmlnode in, int type)
{
    tstream ts = (tstream)(t->data);
    ehandler eh = (ehandler)(ts->data_aux1);

    if(type == XMLSTREAM_INIT)
    {
        pth_cleanup_push(netout_cleanup, (void *)t);
        return;
    }

    if(type == XMLSTREAM_ROOT)
    {
        if(j_strcmp(xmlnode_get_attrib(in, "xmlns"), eh->namespace) != 0)
            xmlstream_error(t,"Invalid Namespace.");
        return;
    }

    if(type == XMLSTREAM_ERROR)
    {
        xmlstream_error(t,"You sent invalid XML.");
        return;
    }

    if(type == XMLSTREAM_NODE)
    {
        if(j_strcmp(xmlnode_get_name(in),"stream:error") == 0)
        {
            log_error("etherxd","remote server XML Stream error: %s",xmlnode_get_data(xmlnode_get_firstchild(in)));
            pth_exit(NULL);
            return;
        }
        log_warn("etherxd","receiving data on connection to remote server where we shouldn't be");
    }
}

void netout_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    ehandler eh = (ehandler)(ts->data_aux1);

    log_debug(ZONE,"netout_cleanup");

    logger_conn2(eh->namespace, eh->host, ts->flag_aux1);
    connections = ehandler_list_free(connections, eh);
}

void netout_write(thread t, epacket e)
{
    tstream ts = (tstream)(t->data);
    tstream_buffer b;
 
    log_debug(ZONE,"net_write");
 
    if(t == NULL || e == NULL)
        return;
 
    b = pmalloc(xmlnode_pool(e->x),sizeof(struct tstream_buffer_struct));
    b->data = xmlnode2str(epacket_serialize(e));
    b->data_sz = strlen(b->data);
    b->sent = 0;
    b->obj = (void *)e;
    b->p = xmlnode_pool(e->x);

    if(!ts->flag1) /* store out of band if the stream hasn't fully opened both ways yet */
        tstream_write_buffer(t, b);
    else
        tstream_oob_buffer(t, b);
}

void netout_error(thread t, tstream_buffer buffer)
{
    /* the stream is closed and these are the epackets that didn't get sent */
    epacket e, eold = (epacket)(buffer->obj);
 
    log_debug(ZONE,"net_error");
 
    if(buffer != NULL && eold != NULL)
    {
        e = epacket_new(xmlnode_dup(eold->x),eold->to, eold->from, eold->ns);
        log_error("etherxd","bouncing data intended for %s",e->to);
        epacket_error(e,"delivery failure");
        manager_send(e);
    }
}

ehandler netout_connect(char *host, char *namespace)
{
    ehandler eh;
    tstream ts;

    eh = _ehandler_new(host, namespace);

    /* create a special new connection! xmlstream can parse it, but we'll handle the buffer */
    eh->t =  tstream_connect(5269, host, _xmlstream_main, netout_error, netout_onNode);
    xmlstream_write_header(eh->t, xmlstream_header(namespace, host, NULL));
    ts = (tstream)(eh->t->data);
    ts->data_aux1 = (void *)eh;

    connections = ehandler_list_add(connections, eh);
}
